package de.tud.kom.p2psim.api.resourcedistribution;

import de.tud.kom.p2psim.api.common.SimHost;

/**
 * Listeners registered at a {@link ResourceModel} get notified of changes in resource units.
 *
 * @author Tim Feuerbach
 */
public interface ResourceChangeListener {
    /**
     * Notifies about a change in resources.
     * <p/>
     * If newValue is 0 and the resource is vital, the host is assumed to be offline.
     *
     * @param host           the host which has its resources changed
     * @param resourceTypeId the unique resource type identifier
     * @param oldValue       the old resource unit count
     * @param newValue       the new resource unit count; newValue &lt; oldValue means consumption, newValue &gt;
     *                       oldValue
     *                       a refill
     * @param vital          whether the node's existence depends on this resource being &gt; 0
     */
    public void resourceUnitsChanged(SimHost host, short resourceTypeId, double oldValue, double newValue, boolean
            vital);
}
