package de.tud.kom.p2psim.impl.analyzer.metric.output;

import de.tudarmstadt.maki.simonstrator.api.Host;
import de.tudarmstadt.maki.simonstrator.api.Oracle;
import de.tudarmstadt.maki.simonstrator.api.Time;
import de.tudarmstadt.maki.simonstrator.api.common.metric.ActiveMetric;
import de.tudarmstadt.maki.simonstrator.api.common.metric.Metric;

import java.io.BufferedWriter;
import java.io.IOException;
import java.io.Writer;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;

/**
 * Writes metric output to CSV files
 *
 * @author Tim Feuerbach
 */
public class MetricOutputCSV extends AbstractOutput implements ActiveMetric.ActiveMetricListener {
    String path;

    Map<ActiveMetric, Writer> writers = new HashMap<>();

    @Override
    public void onInitialize(List<Metric> metrics) {
        Path basePath = Paths.get(path.replace("*", "x"));
        basePath.toFile().mkdirs();

        for (Metric metric : metrics) {
            if (metric instanceof ActiveMetric) {
                ActiveMetric am = (ActiveMetric) metric;
                am.addActiveMetricListener(this);
                try {
                    BufferedWriter writer = Files.newBufferedWriter(basePath.resolve(metric.getName() + ".csv"),
                            StandardCharsets.UTF_8);
                    writer.write("#time;host;value\n");
                    writers.put(am, writer);

                } catch (IOException e) {
                    throw new RuntimeException("Can't write to CSV file", e);
                }
            }


        }
    }

    @Override
    public void onStop() {
        for (ActiveMetric metric : new HashSet<>(writers.keySet())) {
            try {
                writers.get(metric).close();
                writers.remove(metric);
            } catch (IOException e) {
                throw new RuntimeException("Could not close metric writer", e);
            }
        }
    }

    public void setPath(String path) {
        this.path = path;
    }

    @Override
    public void onMetricUpdate(ActiveMetric<?> activeMetric) {
        long time = Time.getCurrentTime();

        String hostID;
        if (activeMetric.isOverallMetric()) {
            writeMetricValue(activeMetric, time, "-1", activeMetric.getOverallMetric());
        } else {
            for (Host host : Oracle.getAllHosts()) {
                writeMetricValue(activeMetric, time, host.getId().toString(), activeMetric.getPerHostMetric(host.getId()));
            }
        }
    }

    private void writeMetricValue(ActiveMetric<?> activeMetric, long time, String host, Metric.MetricValue metricValue) {
        if (metricValue != null) {
            Object value = metricValue.getValue();
            if (metricValue.isValid()) {
                if (value instanceof Number) {
                    double vd = ((Number) value).doubleValue();
                    if (Double.isNaN(vd)) {
                        return;
                    }
                }
                try {
                    Writer writer = writers.get(activeMetric);
                    if (writer != null) {
                        writer.write(String.valueOf(time));
                        writer.write(";");
                        writer.write(host);
                        writer.write(";");
                        writer.write(value.toString());
                        writer.write("\n");
                    }
  
                } catch (IOException e) {
                    throw new RuntimeException("Could not write CSV output", e);
                }
            }
        }
    }
}
