/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 * 
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tud.kom.p2psim.impl.churn;

import java.io.IOException;
import java.nio.charset.Charset;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;
import java.util.Arrays;
import java.util.LinkedList;

import de.tud.kom.p2psim.impl.churn.views.ListView;
import de.tud.kom.p2psim.impl.simengine.Simulator;
import de.tudarmstadt.maki.simonstrator.api.Time;

public class ChurnEventScheduler implements Runnable {
	
	private MaxPeerCountChurnGeneratorSlider maxPeerCountChurnGeneratorSlider;
	private ListView listView = null;
	
	private int currentElement = 0;
	private LinkedList<ChurnInfo> executedChurns = new LinkedList<>();

	public ChurnEventScheduler(MaxPeerCountChurnGeneratorSlider maxPeerCountChurnGeneratorSlider) {
		super();
		this.maxPeerCountChurnGeneratorSlider = maxPeerCountChurnGeneratorSlider;
		
		Thread t = new Thread(this);
		t.start();
	}
	
	public int getCurrentElement() {
		return currentElement;
	}
	
	public void setListView(ListView listView) {
		if(listView != null) {
			this.listView = listView;
		}
	}

	@Override
	public void run() {
		while(true) {
			// Wait until listView is initiated.
			if(listView == null) {
				continue;
			}
			if(!maxPeerCountChurnGeneratorSlider.hasEventsInQueue()) {
				if((listView.getModel().size() - 1) >= currentElement) {
					// No Event scheduled and enough elements in list
					System.out.println(listView.getModel().getElementAt(currentElement).toString());
					ChurnInfo nextChurn = listView.getModel().getElementAt(currentElement);
					maxPeerCountChurnGeneratorSlider.setBurst(nextChurn);
					nextChurn.setStartTime(Simulator.getCurrentTime());
					this.executedChurns.add(nextChurn);
					currentElement++;
					listView.setCurrentElement(currentElement);
				}
			}
		}
	}
	
	public void saveExecutedChurnEvents() {
		Path file = Paths.get(System.getProperty("user.dir") + System.getProperty("file.separator")
			+ "config" + System.getProperty("file.separator") 
			+ "adaptivemonitoring" + System.getProperty("file.separator")
			+ "csvchurnfiles" + System.getProperty("file.separator")
			+ "slider_" + System.currentTimeMillis() + ".csv");
		System.out.println("Saving churn to: " + file.toString());
		try {
			Files.createFile(file);
			Files.write(file, Arrays.asList("#startTime, churnJoinLeaveLength, numberOfClients -- minimum of 10 minutes between the bursts!"), Charset.forName("UTF-8"), StandardOpenOption.APPEND);
			for(ChurnInfo churn: this.executedChurns) {
				Files.write(file, Arrays.asList(churn.getStartTime() / Time.MINUTE + "m, " + churn.getBurstLength() / Time.MINUTE + "m, " + churn.getNumberOfClients()), Charset.forName("UTF-8"), StandardOpenOption.APPEND);
			}
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
	}
}
