/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 * 
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tud.kom.p2psim.impl.churn;

import de.tudarmstadt.maki.simonstrator.api.Time;

/**
 * Churn Info for the fluctuation intervals.
 * 
 * @author Nils Richerzhagen
 */
public class ChurnInfo {

	/**
	 * The time the burst takes.
	 */
	private long burstLength;

	/**
	 * The max number of nodes that join during that burst.
	 */
	private int numberOfClients;
	
	private long startTime;

	public ChurnInfo(long burstLength,
			int numberOfClients) {
		this.burstLength = burstLength;
		this.numberOfClients = numberOfClients;
		this.startTime = 0;
	}
	
	public void setStartTime(long startTime) {
		this.startTime = startTime;
	}

	public int getNumberOfClients() {
		return numberOfClients;
	}

	public long getBurstLength() {
		return burstLength;
	}
	
	public long getStartTime() {
		return startTime;
	}
	
	
	@Override
	public String toString() {
		return "Clients: " + numberOfClients + " peers | Burst: " + (burstLength / Time.MINUTE) + " m";
	}
}
