/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 * 
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tud.kom.p2psim.impl.churn.views;

import java.awt.GridLayout;
import java.awt.Label;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.tud.kom.p2psim.impl.churn.ChurnInfo;
import de.tudarmstadt.maki.simonstrator.api.Time;
import de.tudarmstadt.maki.simonstrator.api.component.GlobalComponent;

public class SliderView extends JFrame implements GlobalComponent, Runnable {
	
	private static final int MIN_BURST_LENGTH = 10;  // Minutes
	private static final int MAX_BURST_LENGTH = 100; // Minutes
	
	Label timeValue;
	Label peerLabel;
	ListView listView;
	JSlider timeSlider;
	JSlider peerSlider;
	
	private int maxNumberOfNodes; 
	
	public SliderView(int maxNumberOfNodes) {
		super();
		
		this.maxNumberOfNodes = maxNumberOfNodes;
		
		Thread t = new Thread(this);
		t.start();
	}
	
	@Override
	public void run() {
		// Setup list of events
		this.listView = new ListView();
		this.listView.setVisible(true);
		
		// Setup slider view
		this.setTitle("Churn Configurator");
		this.setLayout(new GridLayout(7,1));
		
		// Draw label for current peer value
		this.peerLabel = new Label("0 peers");
		this.peerLabel.setAlignment(Label.CENTER);

		// Draw slider for peers
		peerSlider = new JSlider(0, this.maxNumberOfNodes, 1);
		peerSlider.setName("amountslider");
		peerSlider.setPaintLabels(true);
		peerSlider.setMajorTickSpacing(110);
		peerSlider.setMinorTickSpacing(11);
		peerSlider.setPaintTicks(true);

		// Assign listener for peer change
		peerSlider.addChangeListener(new ChangeListener() {
			
			@Override
			public void stateChanged(ChangeEvent e) {
				// Update peer label on change
				JSlider source = (JSlider) e.getSource();
				peerLabel.setText(Integer.toString(source.getValue()) + " peers");
			}
		});
		
		this.timeValue = new Label("0 m");
		this.timeValue.setAlignment(Label.CENTER);
		
		timeSlider = new JSlider(MIN_BURST_LENGTH, MAX_BURST_LENGTH);
		timeSlider.setName("timeslider");
		timeSlider.setPaintLabels(true);
		timeSlider.setMajorTickSpacing(20);
		timeSlider.setMinorTickSpacing(2);
		timeSlider.setPaintTicks(true);

		timeSlider.addChangeListener(new ChangeListener() {
			
			@Override
			public void stateChanged(ChangeEvent e) {
				// Update time label on change
				JSlider source = (JSlider) e.getSource();
				timeValue.setText(Integer.toString(source.getValue()) + " m");
			}
		});
		
		// Draw heading for peer slider
		Label temp = new Label("Amount of Peers");
		temp.setAlignment(Label.CENTER);
	
		// Add components to view
		this.add(temp);
		this.add(peerSlider);
		this.add(this.peerLabel);
		
		// Draw heading for time slider
		temp = new Label("Timespan");
		temp.setAlignment(Label.CENTER);

		// Add components to view
		this.add(temp);
		this.add(timeSlider);
		this.add(this.timeValue);
		
		// Draw and add button for adding new events to queue
		JButton addButton = new JButton("Add to list");
		addButton.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				if(timeSlider.getValue() != 0 && peerSlider.getValue() != 0) {
					listView.addChurn(new ChurnInfo(timeSlider.getValue() * Time.MINUTE, peerSlider.getValue()));
				}
			}
		});
		this.add(addButton);
		
		// Setup window
		this.setSize(300, 300);
		this.setDefaultCloseOperation(EXIT_ON_CLOSE);
		this.setVisible(true);
		timeValue.setText(Integer.toString(timeSlider.getValue()) + " m");
		peerLabel.setText(Integer.toString(peerSlider.getValue()) + " peers");
	}
	
	public ListView getListView() {
		return this.listView;
	}
}