/*
 * Copyright (c) 2005-2010 KOM - Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 * 
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tud.kom.p2psim.impl.topology.movement.modularosm.attraction;

import de.tud.kom.p2psim.impl.topology.PositionVector;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.location.AttractionPoint;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.location.LocationActuator;

/**
 * An attraction point that can be used to assign a target of a {@link LocationActuator}.
 * Unlike a {@link AttractionPointImpl}, this point is not uniquely named.
 * 
 * @author Tim Feuerbach
 * @version 1.0, 30.01.2017
 */
public class TargetAssignmentAttractionPoint extends PositionVector implements AttractionPoint {
	
	public TargetAssignmentAttractionPoint(PositionVector targetLocation) {
		super(targetLocation);
	}
	
	public TargetAssignmentAttractionPoint(double longitude, double latitude) {
		super(new PositionVector(longitude, latitude));
	}

	@Override
	public String getName() {
		return "Generic target";
	}

	@Override
	public double getWeight() {
		return 0;
	}

	@Override
	public double getRadius() {
		return 0;
	}

	@Override
	public void setWeight(double weight) {
		throw new UnsupportedOperationException("Generic points cannot be modified");
	}

	@Override
	public void setRadius(double radius) {
		throw new UnsupportedOperationException("Generic points cannot be modified");
	}

	@Override
	public AttractionPoint clone(String newName) {
		return new TargetAssignmentAttractionPoint(this);
	}

	

}
